@@ -1,16 +1,15 @@
 import concurrent.futures
 import logging
+import os
 import threading
 
-import click
 import boto3
-import os
-
+import click
+import pathspec
+from boto3.s3.transfer import TransferConfig
 from tqdm import tqdm
-from boto3.s3.transfer import TransferConfig, S3Transfer
 
 from tensorkube.constants import REGION
-from multiprocessing import Pool
 
 
 # NOTE: bucket name must be universally unique
@@ -20,6 +19,7 @@ def create_s3_bucket(bucket_name, region=REGION):
         s3_client = boto3.client('s3', region_name=region)
         if region is None or region == REGION:
             s3_client.create_bucket(Bucket=bucket_name)
+            click.echo(f"Created bucket {bucket_name} in region {region}")
         else:
             location = {'LocationConstraint': region}
             s3_client.create_bucket(Bucket=bucket_name, CreateBucketConfiguration=location)
@@ -30,7 +30,7 @@ def create_s3_bucket(bucket_name, region=REGION):
     return True
 
 
-def list_s3_buckets(region: str=REGION):
+def list_s3_buckets(region: str = REGION):
     try:
         s3 = boto3.client('s3', region_name=region)
         response = s3.list_buckets()
@@ -63,24 +63,34 @@ def upload_file(file_name, bucket_name, s3_key, s3_client, progress_bar):
 
     config = boto3.s3.transfer.TransferConfig(use_threads=False)
     try:
-        s3_client.upload_file(file_name, bucket_name, s3_key, Callback=upload_progress, Config=config)
-        # logging.info(f"Successfully uploaded {file_name} to {bucket_name}/{s3_key}")
+        s3_client.upload_file(file_name, bucket_name, s3_key, Callback=upload_progress,
+                              Config=config)  # logging.info(f"Successfully uploaded {file_name} to {bucket_name}/{s3_key}")
     except Exception as e:
         logging.error(f"Error uploading {file_name}: {e}")
     finally:
         progress_bar.close()
 
 
+def load_dockerignore(file_path):
+    with open(file_path, 'r') as f:
+        patterns = f.read().splitlines()
+    return pathspec.PathSpec.from_lines(pathspec.patterns.GitWildMatchPattern, patterns)
+
+
 def upload_files_in_parallel(bucket_name, folder_path, region='us-east-1', s3_path=''):
     s3_client = boto3.client('s3', region_name=region)
     files_to_upload = []
 
+    dockerignore_filepath = os.path.join(folder_path, '.dockerignore')
+    if os.path.exists(dockerignore_filepath):
+        spec = load_dockerignore(dockerignore_filepath)
+    else:
+        spec = pathspec.PathSpec([])
+
     for root, dirs, files in os.walk(folder_path):
         for file in files:
-            if file.startswith('.') or file == ".DS_Store":
-                continue
             local_file = os.path.join(root, file)
-            if '/.' in local_file:
+            if spec.match_file(local_file):
                 continue
             files_to_upload.append(local_file)
 
@@ -119,6 +129,19 @@ def empty_s3_bucket(bucket_name, region=REGION):
 
     return True
 
+
+def empty_s3_folder(bucket_name, folder_name, region=REGION):
+    try:
+        s3 = boto3.resource('s3', region_name=region)
+        bucket = s3.Bucket(bucket_name)
+        bucket.objects.filter(Prefix=folder_name).delete()
+    except Exception as e:
+        click.echo(f"Error emptying folder {folder_name} in bucket {bucket_name} in region {region}: {e}")
+        return False
+
+    return True
+
+
 def delete_s3_bucket(bucket_name, region=REGION):
     try:
         click.echo(f"Emptying bucket {bucket_name}...")
@@ -126,6 +149,7 @@ def delete_s3_bucket(bucket_name, region=REGION):
         click.echo(f"Deleting bucket {bucket_name}...")
         s3 = boto3.client('s3', region_name=region)
         s3.delete_bucket(Bucket=bucket_name)
+        click.echo(f"Deleted bucket {bucket_name} in region {region}")
     except Exception as e:
         click.echo(f"Error deleting bucket {bucket_name} in region {region}: {e}")
         return False
